@@ -76,7 +76,9 @@ Fraction PipeWireProduce::maxFramerate() const
 void PipeWireProduce::setMaxFramerate(const Fraction &framerate)
 {
     m_maxFramerate = framerate;
-    m_stream->setMaxFramerate(framerate);
+    if (m_stream) {
+        m_stream->setMaxFramerate(framerate);
+    }
 }
 
 int PipeWireProduce::maxPendingFrames() const
@@ -115,8 +117,8 @@ void PipeWireProduce::setupStream()
     m_passthroughThread = std::thread([this]() {
         m_passthroughRunning = true;
         while (m_passthroughRunning) {
-            std::unique_lock<std::mutex> lock(m_frameReceivedMutex);
-            m_frameReceivedCondition.wait(lock);
+            std::unique_lock<std::mutex> lock(m_passthroughMutex);
+            m_passthroughCondition.wait(lock);
 
             if (!m_passthroughRunning) {
                 break;
@@ -126,7 +128,7 @@ void PipeWireProduce::setupStream()
             m_pendingFilterFrames -= filtered;
             m_pendingEncodeFrames += queued;
 
-            m_frameReceivedCondition.notify_all();
+            m_outputCondition.notify_all();
         }
     });
     pthread_setname_np(m_passthroughThread.native_handle(), "PipeWireProduce::passthrough");
@@ -134,8 +136,8 @@ void PipeWireProduce::setupStream()
     m_outputThread = std::thread([this]() {
         m_outputRunning = true;
         while (m_outputRunning) {
-            std::unique_lock<std::mutex> lock(m_frameReceivedMutex);
-            m_frameReceivedCondition.wait(lock);
+            std::unique_lock<std::mutex> lock(m_outputMutex);
+            m_outputCondition.wait(lock);
 
             if (!m_outputRunning) {
                 break;
@@ -174,13 +176,13 @@ void PipeWireProduce::destroy()
 
     if (m_passthroughThread.joinable()) {
         m_passthroughRunning = false;
-        m_frameReceivedCondition.notify_all();
+        m_passthroughCondition.notify_all();
         m_passthroughThread.join();
     }
 
     if (m_outputThread.joinable()) {
         m_outputRunning = false;
-        m_frameReceivedCondition.notify_all();
+        m_outputCondition.notify_all();
         m_outputThread.join();
     }
 
@@ -244,7 +246,7 @@ void PipeWireProduce::processFrame(const PipeWireFrame &frame)
     m_pendingFilterFrames++;
     m_previousPts = pts;
 
-    m_frameReceivedCondition.notify_all();
+    m_passthroughCondition.notify_all();
 }
 
 void PipeWireProduce::stateChanged(pw_stream_state state)
@@ -269,46 +271,19 @@ void PipeWireProduce::stateChanged(pw_stream_state state)
 
 std::unique_ptr<Encoder> PipeWireProduce::makeEncoder()
 {
-    auto encoderType = m_encoderType;
-    bool forceSoftware = false;
-    bool forceHardware = false;
-
-    if (qEnvironmentVariableIsSet("KPIPEWIRE_FORCE_ENCODER")) {
-        auto forcedEncoder = qEnvironmentVariable("KPIPEWIRE_FORCE_ENCODER");
-        if (forcedEncoder == u"libvpx") {
-            qCWarning(PIPEWIRERECORD_LOGGING) << "Forcing VP8 Software encoding";
-            encoderType = PipeWireBaseEncodedStream::VP8;
-            forceSoftware = true;
-        } else if (forcedEncoder == u"libvpx-vp9") {
-            qCWarning(PIPEWIRERECORD_LOGGING) << "Forcing VP9 Software encoding";
-            encoderType = PipeWireBaseEncodedStream::VP9;
-            forceSoftware = true;
-        } else if (forcedEncoder == u"libx264") {
-            qCWarning(PIPEWIRERECORD_LOGGING) << "Forcing H264 Software encoding, main profile";
-            encoderType = PipeWireBaseEncodedStream::H264Main;
-            forceSoftware = true;
-        } else if (forcedEncoder == u"h264_vaapi") {
-            qCWarning(PIPEWIRERECORD_LOGGING) << "Forcing H264 Hardware encoding, main profile";
-            encoderType = PipeWireBaseEncodedStream::H264Main;
-            forceHardware = true;
-        } else if (forcedEncoder == u"libx264_baseline") {
-            qCWarning(PIPEWIRERECORD_LOGGING) << "Forcing H264 Software encoding, baseline profile";
-            encoderType = PipeWireBaseEncodedStream::H264Baseline;
-            forceSoftware = true;
-        } else if (forcedEncoder == u"h264_vaapi_baseline") {
-            qCWarning(PIPEWIRERECORD_LOGGING) << "Forcing H264 Hardware encoding, baseline profile";
-            encoderType = PipeWireBaseEncodedStream::H264Baseline;
-            forceHardware = true;
-        }
+    auto forcedEncoder = qEnvironmentVariable("KPIPEWIRE_FORCE_ENCODER");
+    if (!forcedEncoder.isNull()) {
+        qCWarning(PIPEWIRERECORD_LOGGING) << "Forcing encoder to" << forcedEncoder;
     }
 
     auto size = m_stream->size();
 
-    switch (encoderType) {
+    switch (m_encoderType) {
     case PipeWireBaseEncodedStream::H264Baseline:
     case PipeWireBaseEncodedStream::H264Main: {
         auto profile = m_encoderType == PipeWireBaseEncodedStream::H264Baseline ? Encoder::H264Profile::Baseline : Encoder::H264Profile::Main;
-        if (!forceSoftware) {
+
+        if (forcedEncoder.isNull() || forcedEncoder == u"h264_vaapi") {
             auto hardwareEncoder = std::make_unique<H264VAAPIEncoder>(profile, this);
             hardwareEncoder->setQuality(m_quality);
             hardwareEncoder->setEncodingPreference(m_encodingPreference);
@@ -317,7 +292,7 @@ std::unique_ptr<Encoder> PipeWireProduce::makeEncoder()
             }
         }
 
-        if (!forceHardware) {
+        if (forcedEncoder.isNull() || forcedEncoder == u"libx264") {
             auto softwareEncoder = std::make_unique<LibX264Encoder>(profile, this);
             softwareEncoder->setQuality(m_quality);
             softwareEncoder->setEncodingPreference(m_encodingPreference);
@@ -327,7 +302,7 @@ std::unique_ptr<Encoder> PipeWireProduce::makeEncoder()
         }
 
         // Try libopenh264 last, it's slower and has less features.
-        if (!forceHardware && !forceSoftware) {
+        if (forcedEncoder.isNull() || forcedEncoder == u"libopenh264") {
             auto softwareEncoder = std::make_unique<LibOpenH264Encoder>(profile, this);
             softwareEncoder->setQuality(m_quality);
             softwareEncoder->setEncodingPreference(m_encodingPreference);
@@ -338,7 +313,7 @@ std::unique_ptr<Encoder> PipeWireProduce::makeEncoder()
         break;
     }
     case PipeWireBaseEncodedStream::VP8: {
-        if (!forceHardware) {
+        if (forcedEncoder.isNull() || forcedEncoder == u"libvpx") {
             auto encoder = std::make_unique<LibVpxEncoder>(this);
             encoder->setQuality(m_quality);
             if (encoder->initialize(size)) {
@@ -348,7 +323,7 @@ std::unique_ptr<Encoder> PipeWireProduce::makeEncoder()
         break;
     }
     case PipeWireBaseEncodedStream::VP9: {
-        if (!forceHardware) {
+        if (forcedEncoder.isNull() || forcedEncoder == u"libvpx-vp9") {
             auto encoder = std::make_unique<LibVpxVp9Encoder>(this);
             encoder->setQuality(m_quality);
             if (encoder->initialize(size)) {
