@@ -13,6 +13,7 @@
 
 class QQmlApplicationEngine;
 class QTimer;
+class QScreen;
 class Screencasting;
 
 namespace KWayland {
@@ -29,28 +30,43 @@ class OrgFreedesktopPortalScreenCastInterface;
 class PlasmaRecordMe : public QObject
 {
     Q_OBJECT
+    Q_PROPERTY(QRect region READ region NOTIFY regionChanged)
 public:
-    PlasmaRecordMe(Screencasting::CursorMode cursorMode, const QString &source, QObject *parent = nullptr);
+    PlasmaRecordMe(Screencasting::CursorMode cursorMode, const QStringList &sources, bool doSelection, QObject *parent = nullptr);
     ~PlasmaRecordMe() override;
 
     void setDuration(int duration);
 
+    QRect region() const;
+    Q_SCRIPTABLE void addWindow(const QVariantList &uuid, const QString &appId);
     Q_SCRIPTABLE void createVirtualMonitor();
+    Q_SCRIPTABLE void requestSelection();
+    Q_SCRIPTABLE void setRegionPressed(const QString &screen, int x, int y);
+    Q_SCRIPTABLE void setRegionMoved(const QString &screen, int x, int y);
+    Q_SCRIPTABLE void setRegionReleased(const QString &screen, int x, int y);
 
 Q_SIGNALS:
     void cursorModeChanged(Screencasting::CursorMode cursorMode);
     void workspaceChanged();
+    void regionFinal(const QRect &region);
+    void regionChanged(const QRect &region);
 
 private:
-    void start(ScreencastingStream* stream);
+    void startNode(int node);
+    void start(ScreencastingStream *stream, bool allowDmaBuf);
+    void addScreen(QScreen *screen);
+    void addStream(int nodeid, const QString &displayText, int fd, bool allowDmaBuf);
+    bool matches(const QString &source);
 
     const Screencasting::CursorMode m_cursorMode;
     QTimer* const m_durationTimer;
-    const QString m_sourceName;
-    QVector<std::function<void()>> m_delayed;
+    const QStringList m_sources;
     KWayland::Client::PlasmaWindowManagement* m_management = nullptr;
     Screencasting* m_screencasting = nullptr;
     QQmlApplicationEngine* m_engine;
-    ScreencastingStream *m_workspaceStream = nullptr;
+    QList<ScreencastingStream *> m_workspaceStreams;
+    QList<ScreencastingStream *> m_regionStreams;
     QRect m_workspace;
+
+    QRect m_region;
 };
